<?php

/**
 *
 * mpath-tools - failover and load balancing between multiple ISP made easy
 * 
 * Copyright (C) 2008-2010 Vincent Negrier aka. sIX <six at aegis-corp.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 * @package mpath-tools
 * @subpackage control
 */

namespace MPath_Tools\Control;

use \Nanoserv\Connection_Handler;

class Packet {

	const TYPE_HELLO = 1;
	const TYPE_LOG = 2;
	const TYPE_STOP = 3;
	const TYPE_COMMIT = 4;
	const TYPE_REFRESH = 5;
	const TYPE_RELOAD = 6;
	const TYPE_GATEWAY_LIST = 10;
	const TYPE_GATEWAY_GET = 11;
	const TYPE_GATEWAY_SET = 12;
	const TYPE_RULE_GET = 20;
	const TYPE_RULE_ADD = 21;
	const TYPE_RULE_DEL = 22;
	const TYPE_RULE_SET = 23;
	const TYPE_RULE_INSERT = 24;
	const TYPE_LOG_GET = 30;
	const TYPE_LOG_SET = 31;
	const TYPE_CONTROL_LIST = 40;
	const TYPE_STATUS = 50;

	public $type;
	public $message;

	public function __construct($type, $message) {

		$this->type = $type;
		$this->message = $message;
	
	}

}

abstract class Connection extends Connection_Handler {

	const STATE_READY = 1;
	const STATE_WAIT_DATA = 2;

	private $state = self::STATE_READY;
	private $expect_len = 0;
	private $buf = "";

	abstract protected function on_Read_Pkt(Packet $pkt);
	
	public function on_Read($s) {

		$this->buf .= $s;

		do {
		
			switch ($this->state) {

				case self::STATE_READY:

				if (strlen($this->buf) >= 4) {

					list(,$tmp) = unpack("L", substr($this->buf, 0, 4));
					
					$this->state = self::STATE_WAIT_DATA;
					$this->expect_len = $tmp;
					$this->buf = substr($this->buf, 4);
				
				} else {

					break 2;

				}
				
				case self::STATE_WAIT_DATA:

				if (strlen($this->buf) >= $this->expect_len) {

					$pkt = unserialize(gzinflate(substr($this->buf, 0, $this->expect_len)));

					$this->state = self::STATE_READY;
					$this->buf = substr($this->buf, $this->expect_len);

					$this->on_Read_Pkt(new Packet($pkt["type"], $pkt["message"]));
				
				} else {

					break 2;

				}
				
			}
	
		} while (true);
	
	}

	public function Send($type, $message = false, $callback = false) {

		$pkt = array("type" => $type, "message" => $message);

		$tmp = gzdeflate(serialize($pkt));
		
		$this->Write(pack("L", strlen($tmp)) . $tmp, $callback);
	
	}

}

?>
