<?php

/**
 *
 * mpath-tools - failover and load balancing between multiple ISP made easy
 * 
 * Copyright (C) 2008-2010 Vincent Negrier aka. sIX <six at aegis-corp.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 * @package mpath-tools
 * @subpackage control
 */

namespace MPath_Tools\Control;

use \Nanoserv\Connection_Handler;

class Client_Options {

	public $agent;
	
	public function __construct($agent = NULL) {

		$this->agent = $agent;
	
	}

}

class Client_Connection extends Connection {

	public $peer;
	public $received = array();

	public $options;

	public function __construct(Client_Options $options) {

		$this->options = $options;
	
	}
	
	public function on_Connect() {

		$this->peer = $this->socket->Get_Peer_Name();
	
		$hello = array("agent" => $this->options->agent);
		
		$this->Send(Packet::TYPE_HELLO, $hello);
	
	}

	public function on_Connect_Fail($code) {

		switch ($code) {

			case Connection_Handler::FAIL_TIMEOUT:		$msg = "timeout";		break;
			case Connection_Handler::FAIL_CONNREFUSED:	$msg = "connection refused";		break;

			default:										$msg = "unknown";		break;

		}
		
		throw new \Exception("connection failed: {$msg}");
	
	}

	protected function on_Read_Pkt(Packet $pkt) {

		$this->Queue_Packet($pkt);

	}

	public function Queue_Packet(Packet $pkt) {

		$this->received[] = $pkt;

	}
	
	public function Has_Data() {

		return (bool)$this->received;

	}
	
	public function Get_Packet() {

		return array_shift($this->received);

	}
	
	public function on_Disconnect() {

	}

}

?>
