<?php

/**
 *
 * mpath-tools - failover and load balancing between multiple ISP made easy
 * 
 * Copyright (C) 2008-2010 Vincent Negrier aka. sIX <six at aegis-corp.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 * @package mpath-tools
 * @subpackage control
 */

namespace MPath_Tools\Control;

use \Nanoserv\Core as Nanoserv;

class Server_Connection extends Connection {

	const DEFAULT_LISTEN_ADDR = "127.0.0.1:7842";
	
	public $peer;
	public $agent;
	public $connect_t;
	public $last_activity_t;

	public $received = array();
	
	static public function Send_Log($message, $level, Server_Connection $except = NULL) {

		$tmp = array("message" => $message, "level" => $level);
		
		self::Send_All(Packet::TYPE_LOG, $tmp, $except);

	}
	
	static public function Get_All() {

		$ret = array();
		
		foreach (Nanoserv::Get_Connections() as $c) {

			if ($c instanceof self) {

				$ret[] = $c;

			}

		}

		return $ret;
	
	}

	static public function Send_All($msg_type, $msg, Server_Connection $except = NULL) {

		foreach (self::Get_All() as $c) {
			
			if (isset($except) && ($c === $except)) continue;
			
			$c->Send($msg_type, $msg);

		}
	
	}
	
	public function on_Accept() {

		$this->connect_t = $this->last_activity_t = time();
		$this->peer = $this->socket->Get_Peer_Name();
	
		$hello = array("server" => "mpathd/" . \MPath_Tools\VERSION);
		
		$this->Send(Packet::TYPE_HELLO, $hello);
	
	}

	protected function on_Read_Pkt(Packet $pkt) {

		$this->last_activity_t = time();
		
		if ($pkt->type === Packet::TYPE_HELLO) {

			$this->agent = $pkt->message["agent"];
		
		}
		
		$this->Queue_Packet($pkt);

	}

	public function Has_Data() {

		return (bool)$this->received;

	}
	
	public function Get_Packet() {

		return array_shift($this->received);

	}

	public function Queue_Packet(Packet $pkt) {

		$this->received[] = $pkt;
	
	}
	
	public function on_Disconnect() {

		\MPath_Tools\mpd_debug("lost control connection from '{$this->peer}'");
	
	}

}

?>
