<?php

/**
 *
 * mpath-tools - failover and load balancing between multiple ISP made easy
 * 
 * Copyright (C) 2008-2010 Vincent Negrier aka. sIX <six at aegis-corp.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 * @package mpath-tools
 * @subpackage log
 */

namespace MPath_Tools;

use \Exception;

class Log {

	const LEVEL_DEBUG = 1;
	const LEVEL_NOTICE = 2;
	const LEVEL_WARNING = 4;
	const LEVEL_ERROR = 8;

	const LEVEL_DEFAULT = 14;
	const LEVEL_ALL = 15;
	
	static private $all = array();
	
	public $filename;
	public $stream;
	public $level = self::LEVEL_DEFAULT;

	public function __construct($filename, $level = NULL) {

		$this->filename = $filename;

		if (isset($level)) {
			
			$this->level = $level;

		}

		if (!$this->stream = @fopen($filename, "a")) {

			throw new Exception("cannot open log file for writing '{$filename}'");

		}
	
	}
	
	private function Do_Fwrite($s) {

		@fwrite($this->stream, $s);
	
	}
	
	static public function Get_All() {

		return self::$all;

	}
	
	static public function Get_By_Filename($filename) {

		foreach (self::Get_All() as $log) {

			if ($log->filename === $filename) {

				return $log;

			}

		}

		throw new Exception("invalid log filename '{$filename}'");

	}
	
	static public function Write($txt, $level = self::LEVEL_NOTICE) {

		$s = date("Ymd:His") . " " . $txt . "\n";

		foreach (self::Get_All() as $log) {

			if ($level & $log->level) {

				$log->Do_Fwrite($s);

			}
		
		}
	
	}

	static public function Level_To_String($level) {

		switch ($level) {

			case self::LEVEL_DEBUG:		return "DEBUG";
			case self::LEVEL_NOTICE:	return "NOTICE";
			case self::LEVEL_WARNING:	return "WARNING";
			case self::LEVEL_ERROR:		return "ERROR";

			default:					return "???";

		}

	}
	
	static public function Initialize($conf) {

		self::$all = array();
		
		foreach ($conf->Get_Elements_By_Name("log") as $clog) {

			if ($clog->disabled) {

				continue;

			}
			
			$level = 0;
			
			foreach ($clog->level as $lvl) {

				if (defined($cname = "self::LEVEL_" . strtoupper($lvl))) {

					$level |= constant($cname);
				
				} else {

					throw new Exception("invalid log level '{$lvl}'");

				}
			
			}
			
			foreach ($clog->Get_Values() as $filename) {

				self::$all[] = new self($filename, $level);
			
			}
		
		}
	
		return self::$all;
	
	}

	public function __destruct() {

		if (is_resource($this->stream)) {
		
			fclose($this->stream);

		}
	
	}

}

?>
