#!/usr/local/bin/php
<?php

function install_dir($dst, $src, $overwrite = true) {

	$dst_dir = $dst . "/" . $src;
	
	if ((!is_dir($dst_dir)) && (!mkdir($dst_dir))) {

		return "cannot create directory {$dst_dir}";

	}
	
	foreach (scandir($src) as $name) {

		if ($name[0] === ".") continue;

		if (is_dir($fname = "{$src}/{$name}")) {

			$tmp = install_dir($dst, $fname, $overwrite);

			if ($tmp !== true) {

				return $tmp;

			}

		} else {

			$dname = $dst_dir . "/" . $name;
			
			if (file_exists($dname) && !$overwrite) {

				continue;

			}
			
			if (!copy($fname, $dname)) {

				return "cannot copy {$fname}";

			}
		
		}
	
	}

	return true;

}

function install_binary($name, $dst_dir = "/usr/bin") {

	if (!copy($name, $dst_name = $dst_dir . "/" . basename($name, ".php"))) {

		return "cannot copy {$name}";

	}

	if (!chmod($dst_name, 0750)) {

		return "cannot set file {$dst_name} to mode 0750";

	}

	return true;

}

echo "mpath-tools installer version 1.0\n";
echo "---------------------------------\n";

echo "checking php version ... ";

if (version_compare($v = phpversion(), "5.3.0alpha3") < 0) {

	die("failed: you need php >= 5.3\n");

} else {

	echo "ok\n";

}

echo "checking php extensions ... ";

$needed = array("pcntl", "posix", "sockets", "zlib");

foreach ($needed as $ext) {

	if (!extension_loaded($ext)) {

		die("php needs the '{$ext}' extension\n");

	}

}

echo "ok\n";

echo "checking include path ... ";

$inc_dir = false;

foreach (explode(":", ini_get("include_path")) as $dir) {

	if ($dir[0] !== ".") {

		$inc_dir = $dir;
		break;

	}

}

if ($inc_dir === false) {

	die("failed\n");

} else {

	echo "ok\n";

}

if (!is_dir($mpt_dir = "{$inc_dir}/mpath-tools")) {

	echo "creating directory {$mpt_dir} ... ";

	if (!mkdir($mpt_dir)) {

		die("failed\n");

	} else {

		echo "ok\n";

	}

}

echo "installing include directory ... ";

if (($tmp = install_dir($mpt_dir, "inc")) === true) {

	echo "ok\n";

} else {

	die("failed: {$tmp}\n");

}

echo "installing examples directory ... ";

if (($tmp = install_dir($mpt_dir, "examples")) === true) {

	echo "ok\n";

} else {

	die("failed: {$tmp}\n");

}

echo "installing configuration directory ... ";

if (($tmp = install_dir("/", "etc", false)) === true) {

	echo "ok\n";

} else {

	die("failed: {$tmp}\n");

}

echo "installing mpathd ... ";

if (($tmp = install_binary("mpathd.php")) === true) {

	echo "ok\n";

} else {

	die("failed: {$tmp}");

}

echo "installing mpath ... ";

if (($tmp = install_binary("mpath.php")) === true) {

	echo "ok\n";

} else {

	die("failed: {$tmp}");

}

echo "installing mping ... ";

if (($tmp = install_binary("mping.php")) === true) {

	echo "ok\n";

} else {

	die("failed: {$tmp}");

}

?>
