#!/usr/local/bin/php53
<?php

/**
 *
 * mpath-tools - failover and load balancing between multiple ISP made easy
 * 
 * Copyright (C) 2008-2010 Vincent Negrier aka. sIX <six at aegis-corp.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 * @package mpath-tools
 * @subpackage mpath-cli
 */

namespace MPath_Tools;

const DEFAULT_CONF_FILENAME = "mpathd.conf";

require "mpath-tools/inc/getopt.php";
require "mpath-tools/inc/config.php";
require "mpath-tools/inc/gateway.php";
require "mpath-tools/inc/rule.php";
require "mpath-tools/inc/log.php";

require "mpath-tools/inc/client.php";

use MPath_Tools\Config\Core as Conf;
use MPath_Tools\Config\Tokenizer as Tokenizer;
use MPath_Tools\Control\Client;

use \Tokenizer\Exception as Tokenizer_Exception;
use \Exception;

error_reporting(0);

function mp_show_help() {

	echo "usage: {$GLOBALS['argv'][0]} [options]\n";
	echo "\n";
	echo "    -c <filename>      configuration file\n";
	echo "    -d <hostname>      connect to this host (default: " . Client::DEFAULT_HOSTNAME . ")\n";
	echo "    -p <port>          connect to this port (default: " . Client::DEFAULT_PORT . ")\n";
	echo "    -v                 show client version and exit\n";
	echo "    -V                 show server version and exit\n";
	echo "    -q                 be quiet\n";
	echo "    -h                 this help page\n";
	echo "\n";

}

function mp_show_online_help($cmd) {

	switch ($cmd) {
	
		case "":
		echo "show      set       add       insert    del       default   activate  disengage\n";
		echo "disable   auto      commit    refresh   dump      reload    stop      exit\n";
		break;

		case "show":
		echo "show gateways   show gateway   show rules   show rule   show control\n";
		echo "show status\n";
		break;

		case "show gateways":
		echo "show gateways\n";
		echo "show the gateways used by mpathd with their status\n";
		break;

		case "show gateway":
		echo "show gateway [stats|full] <gwname>\n";
		echo "show information about gateway <gwname>\n";
		break;

		case "show rules":
		echo "show rules [current]\n";
		echo "show list of routing rules enforced by mpathd\n";
		echo "if current is specified, show current value of resolved hostnames\n";
		break;

		case "show rule":
		echo "show rule <id>\n";
		echo "show information about rule #<id>\n";
		break;

		case "show control":
		echo "show control\n";
		echo "show active control connections\n";
		break;
		
		case "show status":
		echo "show status\n";
		echo "show information about status of mpathd\n";
		break;
		
		case "set":
		echo "set console   set log   set gateway   set autocommit   set rule\n";
		break;

		case "set console":
		echo "set console <log_params>\n";
		echo "used to configure how mpathd logs are displayed in the console.\n";
		echo "log_params use the same directives as the log blocks in mpathd.conf\n";
		echo "example: set console level +debug\n";
		break;

		case "set log":
		echo "set log <filename> <log_params>\n";
		echo "used to configure how mpathd log files are generated.\n";
		echo "log_params use the same directives as the log blocks in mpathd.conf\n";
		echo "example: set log /var/log/mpathd.log level +debug\n";
		break;
		
		case "set gateway":
		echo "set gateway gwname <gw_params>\n";
		echo "sets one or more gateway configuration parameters.\n";
		echo "gw_params use the same directives as the gateway blocks in mpathd.conf\n";
		echo "example: set gateway adsl ip_addr 123.123.123.123 nexthop 10.0.0.1\n";
		break;

		case "set autocommit":
		echo "set autocommit <on|off>\n";
		echo "if autocommit is set (default), 'add', 'del' and 'default' commands are\n";
		echo "commited right after they are received. if autocommit is not set, changes\n";
		echo "are applied when a 'commit' command is received, or as soon as rules and\n";
		echo "tables have to be regenerated.\n";
		echo "example: set autocommit off\n";
		break;
		
		case "set rule":
		echo "set rule [<rule_id>|<match> <value>] [via <routes>] [policy <policy>] [expires <expires>]\n";
		echo "sets new routes, policy or expiration for given rule\n";
		echo "examples: set rule source 192.168.0.10 via adsl expires 10 min\n";
		echo "          set rule 3 via cable\n";
		break;

		case "add":
		echo "add [rule] <match> <value> via <routes> [policy <policy>] [expires <expires>]\n";
		echo "create a new rule\n";
		echo "examples: add source 192.168.0.10 via adsl expires 10 min\n";
		echo "          add tos lowdelay via cable\n";
		break;

		case "insert":
		echo "insert [rule] [<idx>] <match> <value> via <routes> [policy <policy>] [expires <expires>]\n";
		echo "insert a new rule at position idx\n";
		echo "examples: insert source 192.168.0.10 via adsl expires 10 min\n";
		echo "          insert 3 tos lowdelay via cable\n";
		break;
		
		case "del":
		echo "del [rule] <rule_id>|<match> <value>\n";
		echo "removes a rule\n";
		echo "examples: del source 192.168.0.10\n";
		echo "          del 3\n";
		break;

		case "default":
		echo "default via <routes>\n";
		echo "sets the routes or policy for the default rule\n";
		echo "example: default via adsl cable\n";
		break;
		
		case "activate":
		echo "activate <gwname>\n";
		echo "change gateway state to active\n";
		break;
		
		case "disengage":
		echo "disengage <gwname>\n";
		echo "change gateway state to disengaged\n";
		break;
		
		case "disable":
		echo "disable <gwname>\n";
		echo "change gateway state to disabled\n";
		break;
		
		case "auto":
		echo "auto <gwname>\n";
		echo "change gateway state to automatic (probing determines state)\n";
		break;
		
		case "commit":
		echo "commit\n";
		echo "commit changes to the ip routing rules\n";
		break;

		case "refresh":
		echo "refresh\n";
		echo "resolve hostnames in rules to corresponding ip addresses\n";
		break;
		
		case "dump":
		echo "dump rule    dump rules\n";
		break;

		case "dump rules":
		echo "dump rules\n";
		echo "dump all active rules to the console in configuration file format\n";
		break;
		
		case "dump rule":
		echo "dump rule [<rule_id>|<match> <value>]\n";
		echo "dump specified rule to the console in configuration file format\n";
		break;

		case "reload":
		echo "reload\n";
		echo "reload rules and gateway definitions from configuration files\n";
		break;
		
		case "stop":
		echo "stop <message>\n";
		echo "stops the mpathd daemon, if <message> is provided it will be output to\n";
		echo "the server log file(s).\n";
		echo "example: stop upgrading to the latest version\n";
		break;

		case "exit":
		echo "exit\n";
		echo "quit client\n";
		break;
		
		default:
		echo "sorry, no help available for '{$cmd}'\n";

	}

}

function mp_prompt() {

	$uname = posix_uname();
	
	echo "{$uname['nodename']}#mpath> ";

}

function mp_read_input() {

	global $stdin;

	stream_set_blocking($stdin, false);
	
	$rfd = array($stdin);
	$dfd = array();
	
	if (stream_select($rfd, $dfd, $dfd, 0)) {

		$s = rtrim(fgets($stdin, 8192), "\r\n");

		$ret = $s;
		
	} else {

		$ret = false;

	}

	stream_set_blocking($stdin, true);

	return $ret;

}

function mp_show_log($level, $message) {

	global $console_level;

	if (!($level & $console_level)) {

		return false;

	}
	
	switch ($level) {

		case Log::LEVEL_DEBUG:			$lvl = "DEBUG";		break;
		case Log::LEVEL_NOTICE:			$lvl = "NOTICE";	break;
		case Log::LEVEL_WARNING:		$lvl = "WARNING";	break;
		case Log::LEVEL_ERROR:			$lvl = "ERROR";		break;
		
		default:						$lvl = "??";		break;
	
	}

	echo "[{$lvl}] {$message}\n";

}

function mp_parse_cmd($cmd, $keys = array(), $array_keys = array()) {

	$ret = array();
	
	$t = new Tokenizer($cmd, false);
	
	$all_keys = array_merge($keys, $array_keys);
	
	while ($tok = $t->Get_Token()) {

		if ((!$tok->Is_Key()) || (!in_array($tok->token, $all_keys))) {

			throw new Exception("parse error: unexpected {$tok->token}");

		}

		$val = array();

		while (($tmp = $t->Get_Token(Tokenizer::PEEK)) && ($tmp->Is_Value())) {

			$val[] = $t->Get_Token()->Get_Value();
		
		}

		if (in_array($tok->token, $array_keys)) {

			if (!isset($ret[$tok->token])) {

				$ret[$tok->token] = array();

			}
			
			$ret[$tok->token][] = $val;
		
		} else {
		
			if (isset($ret[$tok->token])) {

				throw new Exception("parse error: duplicate {$tok->token}");

			}
			
			$ret[$tok->token] = $val;

		}

	}

	return $ret;

}

function mp_cmd_set($arg) {

	$cmd = strtok($arg, " ");
	$arg2 = strtok("");

	switch ($cmd) {

		case "console":		mp_cmd_set_console($arg2);		break;
		case "log":			mp_cmd_set_log($arg2);			break;
		case "gateway":		mp_cmd_set_gateway($arg2);		break;
		case "autocommit":	mp_cmd_set_autocommit($arg2);	break;
		case "rule":		mp_cmd_set_rule($arg2);			break;

		default:			echo "set: invalid argument '{$cmd}'\n";

	}

}

function mp_cmd_set_console($arg) {

	global $console_level;
	
	$old_level = $console_level;
	
	mp_do_set_log($arg, $console_level);

	echo "console level set to {$console_level} (was {$old_level})\n";

}

function mp_cmd_set_log($arg) {

	global $ctl;
	
	$filename = trim(strtok($arg, " "));
	$arg2 = trim(strtok(""));

	$log = $ctl->Get_Log($filename);
	$level = $log["level"];

	mp_do_set_log($arg2, $level);

	$ctl->Set_Log(array("filename" => $filename, "level" => $level));

	echo "log level set to {$level} (was {$log['level']})\n";

}

function mp_do_set_log($arg, &$log_level) {

	$cmd = mp_parse_cmd($arg, array("T_LEVEL", "T_DISABLED"));

	if (!$cmd["T_LEVEL"]) {

		throw new Exception("you must specify a level, see 'help set log'");
	
	}
	
	$newlvl = $log_level;

	foreach ($cmd["T_LEVEL"] as $lvl_s) {

		if (is_int($lvl_s)) {

			$lvl = $lvl_s;

		} else {
			
			if ($lvl_s[0] === "+") {

				$op = "+";
				$lvl_s = substr($lvl_s, 1);

			} else if ($lvl_s[0] === "-") {

				$op = "-";
				$lvl_s = substr($lvl_s, 1);

			} else {

				$op = "=";

			}
			
			switch ($lvl_s) {

				case "none":		$lvl = 0;									break;
				case "debug":		$lvl = Log::LEVEL_DEBUG;					break;
				case "notice":		$lvl = Log::LEVEL_NOTICE;					break;
				case "warning":		$lvl = Log::LEVEL_WARNING;					break;
				case "error":		$lvl = Log::LEVEL_ERROR;					break;
				case "all":			$lvl = Log::LEVEL_ALL;						break;

				default:			throw new Exception("invalid log level '{$lvl_s}'");
		
			}
		
		}

		switch ($op) {
		
			case "=":	$newlvl = $lvl;		break;
			case "+":	$newlvl |= $lvl;	break;
			case "-":	$newlvl &= ~$lvl;	break;

		}
	
	}
	
	$log_level = $newlvl;

	return true;

}

function mp_cmd_set_gateway($arg) {

	global $ctl, $autocommit;
	
	$keys = array("T_GATEWAY", "T_IP_ADDR", "T_NETWORK", "T_NEXTHOP", "T_INTERFACE", "T_WEIGHT", "T_INTERVAL", "T_TIMEOUT", "T_STATE", "T_LATENCY");
	$args = mp_parse_cmd("gateway {$arg}", $keys);
	$upd = array();

	foreach ($args as $key => $val) {

		if (count($val) > 1) {

			throw new Exception("{$key} requires only one parameter");
		
		}

		switch ($key) {

			case "T_GATEWAY":	$gwname = $val[0];				break;

			case "T_IP_ADDR":	$upd["ip_addr"] = $val[0];		break;
			case "T_NETWORK":	$upd["network"] = $val[0];		break;
			case "T_NEXTHOP":	$upd["nexthop"] = $val[0];		break;
			case "T_INTERFACE":	$upd["interface"] = $val[0];	break;
			case "T_WEIGHT":	$upd["weight"] = $val[0];		break;
			case "T_INTERVAL":	$upd["interval"] = $val[0];		break;
			case "T_TIMEOUT":	$upd["timeout"] = $val[0];		break;
			case "T_STATE":		$upd["state"] = $val[0];		break;
			case "T_LATENCY":	$upd["latency"] = $val[0];		break;
		
		}
	
	}
	
	$ctl->Set_Gateway($gwname, $upd);

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_set_autocommit($arg) {

	switch (strtolower($arg)) {

		case "":
		case "on":	$newval = true;		break;

		case "off":	$newval = false;	break;
	
	}

	if (!isset($newval)) {

		echo "set autocommit: unknown value '{$arg}' (on|off)\n";

	} else {

		$GLOBALS["autocommit"] = $newval;

		echo "rule autocommit is now " . ($newval ? "on" : "off") . "\n";

	}

}

function mp_cmd_activate($arg) {

	global $ctl, $autocommit;
	
	foreach (explode(" ", $arg) as $gwname) {

		if (($gwname === "gateway") || ($gwname === "gateways")) continue;
		
		$ctl->Set_Gateway($gwname, array("state" => "active"));

	}

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_disengage($arg) {

	global $ctl, $autocommit;
	
	foreach (explode(" ", $arg) as $gwname) {

		if (($gwname === "gateway") || ($gwname === "gateways")) continue;
		
		$ctl->Set_Gateway($gwname, array("state" => "disengaged"));

	}

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_disable($arg) {

	global $ctl, $autocommit;
	
	foreach (explode(" ", $arg) as $gwname) {

		if (($gwname === "gateway") || ($gwname === "gateways")) continue;
		
		$ctl->Set_Gateway($gwname, array("state" => "disabled"));

	}

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_auto($arg) {

	global $ctl, $autocommit;
	
	foreach (explode(" ", $arg) as $gwname) {

		if (($gwname === "gateway") || ($gwname === "gateways")) continue;
		
		$ctl->Set_Gateway($gwname, array("state" => "auto"));

	}

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_show($arg) {

	$cmd = strtok($arg, " ");
	$arg2 = strtok("");

	switch ($cmd) {

		case "gateways":	mp_cmd_show_gateways($arg2);		break;
		case "gateway":		mp_cmd_show_gateway($arg2);			break;
		case "rules":		mp_cmd_show_rules($arg2);			break;
		case "rule":		mp_cmd_show_rule($arg2);			break;
		case "control":		mp_cmd_show_control($arg2);			break;
		case "status":		mp_cmd_show_status($arg2);			break;

		case "default":		switch ($arg2) {

								case "rule":	mp_cmd_show_rule("default");							break 2;
								default:		echo "show default: invalid argument '{$arg2}'\n";		break 2;

							}

		default:			echo "show: invalid argument '{$cmd}'\n";

	}

}

function mp_str_fit($s, $len, $pad_type = STR_PAD_RIGHT) {

	return str_pad(substr($s, 0, $len), $len, " ", $pad_type);

}

function mp_cmd_show_gateways($arg) {

	global $ctl;

	$gws = $ctl->List_Gateways($arg);

	echo "name      nexthop                     state       probe                   %loss\n";
	echo "--------- --------------------------- ----------- ----------------------- -----\n";

	foreach ($gws as $gw) {

		echo mp_str_fit($gw["name"], 9) . " ";
		echo mp_str_fit($gw["nexthop"] . " (" . $gw["interface"] . ")", 27) . " ";
		echo mp_str_fit($gw["state"], 12);
		echo mp_str_fit($gw["status"], 24);

		if ($gw["loss_pct_60sec"] === false) {

			echo "    -";

		} else {
		
			echo mp_str_fit(number_format($gw["loss_pct_60sec"], 2), 5, STR_PAD_LEFT);

		}

		echo "\n";
	
	}

}

function mp_cmd_show_gateway($arg) {

	global $ctl;
	
	$show_params = true;
	$show_stats = false;
	
	if ((substr($arg, 0, 6) === "stats ") || ($arg === "stats")) {

		$show_params = false;
		$show_stats = true;
		$arg = substr($arg, 6);
		$default_all = true;

	} else if ((substr($arg, 0, 5) === "full ") || ($arg === "full")) {

		$show_stats = true;
		$arg = substr($arg, 5);
		$default_all = false;

	}
	
	$tmp = mp_parse_cmd("gateway {$arg}", array("T_GATEWAY"));

	if (!$tmp["T_GATEWAY"] && !$default_all) {

		throw new Exception("you must specify at least one gateway name");

	}

	$gw_data = $ctl->Get_Gateways($tmp["T_GATEWAY"]);

	foreach ($gw_data as $gw) {

		echo "--- gateway " . $gw["name"] . " " . str_repeat("-", 66 - strlen($gw["name"])) . "\n";
		
		if ($show_params) {

			echo "state        {$gw['state']}\n";
			echo "ip_addr      {$gw['ip_addr']}\n";
			echo "network      {$gw['network']}\n";
			echo "interface    {$gw['interface']}\n";
			echo "weight       {$gw['weight']}\n";
			echo "--- probe ---------------------------------------------------------------------\n";
			echo "status       {$gw['status']}\n";

			if ($gw["interval"] !== false) echo "interval     {$gw['interval']}\n";
			if ($gw["timeout"] !== false) echo "timeout      {$gw['timeout']}\n";

			echo "disengage    {$gw['disengage']}\n";
			echo "disable      {$gw['disable']}\n";
			echo "activate     {$gw['activate']}\n";
			echo "latency      {$gw['latency']}\n";

			if ($show_stats) {

				echo "--- statistics ----------------------------------------------------------------\n";

			}

		}
		
		if ($show_stats) {
			
			if ($gw["stats"] === false) {

				echo "not available\n";

			} else {
			
				echo "            10 sec    30 sec     1 min     5 min    10 min    30 min    1 hour\n";

				$stats_def = array("sent", "received", "error", "rtt min", "rtt avg", "rtt max", "rtt mdev", "% loss", "% error");
				
				foreach ($stats_def as $id => $row) {

					$s = mp_str_fit($row, 8) . " ";

					foreach ($gw["stats"] as $col) {

						$s .= mp_str_fit($id > 2 ? number_format($col[$row], 2) : $col[$row], 9, STR_PAD_LEFT) . " ";
					
					}

					$s .= "\n";

					echo $s;
				
				}
		
			}
	
		}
	
	}

}

function mp_rule_match_to_str($match, $short = true) {

	switch ($match) {

		case Rule::MATCH_DEFAULT:		return "default";
		case Rule::MATCH_SOURCE:		return "source";
		case Rule::MATCH_DESTINATION:	return $short ? "dest" : "destination";
		case Rule::MATCH_FWMARK:		return "fwmark";
		case Rule::MATCH_TOS:			return "tos";

		default:						return "??";
	
	}

}

function mp_rule_policy_to_str($policy) {

	switch ($policy) {

		case Rule::DEF_DEFAULT:			return "default";
		case Rule::DEF_UNREACHABLE:		return "unreachable";
		case Rule::DEF_BLACKHOLE:		return "blackhole";
		case Rule::DEF_PROHIBIT:		return "prohibit";

		default:						return "??";

	}

}

function mp_cmd_show_rules($arg) {

	global $ctl;
	
	$current = ($arg == "current");
	
	$rules = $ctl->Get_Rules();

	echo "id  type    matches                                     via                  DE\n";
	echo "--- ------- ------------------------------------------- -------------------- --\n";

	$i = 0;
	
	foreach ($rules as $rule) {

		$s  = mp_str_fit($i, 3, STR_PAD_LEFT) . " ";
		$s .= mp_str_fit(mp_rule_match_to_str($rule["match"]), 7) . " ";
		$s .= mp_str_fit(implode(",", $current ? $rule["current_value"] : $rule["value"]), 43) . " ";

		$tmp = "";
		
		foreach ($rule["routes"] as $route) {

			$tmp .= "[" . implode(",", $route) . "]";
		
		}

		$s .= mp_str_fit($tmp, 20) . " ";

		$s .= ($rule["default_policy"] !== Rule::DEF_DEFAULT) ? "*" : " ";
		$s .= ($rule["expires"] !== false) ? "*" : " ";
		
		$s .= "\n";

		$i++;
		
		echo $s;

	}

}

function mp_cmd_show_rule($arg) {

	global $ctl;
	
	$args = explode(" ", $arg);
	
	if ($current = ($args[0] === "current")) {

		array_shift($args);

		$arg = implode(" ", $args);

	}

	if (($arg === false) || ($arg === "")) {

		throw new Exception("show rule: missing argument");
	
	} else if (is_numeric($arg)) {

		$rules = array_values($ctl->Get_Rules());
		
		if (!isset($rules[$arg])) {

			throw new Exception("rule {$arg} not found");

		}
		
		$rule = array("match" => $rules[$arg]["match"], "value" => $rules[$arg]["value"]);
	
	} else {

		$cmd = mp_parse_cmd($arg, array("T_SOURCE", "T_DESTINATION", "T_FWMARK", "T_TOS", "T_DEFAULT"));

		$rule = mp_parsed_cmd_to_rule($cmd);
	
	}
	
	$rules = $ctl->Get_Rules($rule);
	
	foreach ($rules as $rule) {
		
		$rule["id"] = (int)$rule["id"];

		echo "--- rule #{$rule['id']} " . str_repeat("-", 68 - strlen($rule["id"])) . "\n";
		echo "type         " . mp_rule_match_to_str($rule["match"], false) . "\n";

		foreach ($current ? $rule["current_value"] : $rule["value"] as $k => $v) {
		
			echo ($k ? "             " : "values       ") . $v . "\n";

		}

		foreach ($rule["routes"] as $k => $v) {

			echo ($k ? "             " : "routes       ") . implode(" ", $v) . "\n";
		
		}

		if ($rule["match"] !== Rule::MATCH_DEFAULT) {
			
			echo "             " . mp_rule_policy_to_str($rule["default_policy"]) . "\n";

		}

		if (isset($rule["table"])) {

			echo "table        " . $rule["table"] . "\n";
		
		}

		echo "expires      " . ($rule["expires"] === false ? "never" : date("Y-m-d H:i:s", $rule["expires"])) . "\n";

	}

}

function mp_cmd_show_control() {

	global $ctl;
	
	$controls = $ctl->List_Controls();

	echo "id      agent                 addr:port             connected since     idle\n";
	echo "------- --------------------- --------------------- ------------------- -------\n";
	
	foreach ($controls as $control) {

		$s  = mp_str_fit($control["id"], 7, STR_PAD_LEFT) . " ";
		$s .= mp_str_fit($control["agent"], 21) . " ";
		$s .= mp_str_fit($control["peer"], 21) . " ";
		$s .= mp_str_fit(date("Y-m-d H:i:s", $control["connect_t"]), 19) . " ";
		$s .= mp_str_fit($control["idle_t"], 7, STR_PAD_LEFT);
		$s .= "\n";

		echo $s;
	
	}
	
}

function mp_cmd_show_status() {

	global $ctl;

	$st = $ctl->Get_Status();

	echo "--- status --------------------------------------------------------------------\n";
	echo "version        {$st['version']}\n";
	echo "config file    {$st['config_file']}\n";
	echo "daemon pid     {$st['pid']}\n";
	echo "probe pid      {$st['probe_pid']}\n";
	echo "started        " . date("Y-m-d H:i:s", $st["started_t"]) . "\n";
	
}

function mp_parsed_cmd_to_rule(array $cmd) {

	$ret = array();

	$valid_matches = array("T_SOURCE", "T_DESTINATION", "T_FWMARK", "T_TOS", "T_DEFAULT");

	foreach ($valid_matches as $m_tok) {

		if (isset($cmd[$m_tok])) {

			if (isset($match)) {

				throw new Exception("duplicate match type {$m_tok}");

			}
			
			switch ($m_tok) {

				case "T_SOURCE":		$match = Rule::MATCH_SOURCE;		break;
				case "T_DESTINATION":	$match = Rule::MATCH_DESTINATION;	break;
				case "T_FWMARK":		$match = Rule::MATCH_FWMARK;		break;
				case "T_TOS":			$match = Rule::MATCH_TOS;			break;
				case "T_DEFAULT":		$match = Rule::MATCH_DEFAULT;		break;
			
			}

			$ret["match"] = $match;
			$ret["value"] = $cmd[$m_tok];
		
		}
	
	}

	if (isset($cmd["T_POLICY"])) {

		if (count($cmd["T_POLICY"]) !== 1) {

			throw new Exception("policy takes one parameter");

		}

		$ret["default_policy"] = Rule::Str_To_Policy($cmd["T_POLICY"][0]);
	
	}

	if (isset($cmd["T_EXPIRES"])) {

		if (strtolower($cmd["T_EXPIRES"][0]) === "never") {

			$ret["expires"] = false;

		} else if ($tmp2 = strtotime($tmp = implode(" ", $cmd["T_EXPIRES"]))) {

			$ret["expires"] = $tmp2;
			
		} else {

			throw new Exception("cannot parse expires value '{$tmp}'");

		}

	}

	if (isset($cmd["T_VIA"])) {

		$ret["via"] = $cmd["T_VIA"];

	}
	
	return $ret;

}

function mp_cmd_add($arg) {

	$cmd = strtok($arg, " ");
	$arg2 = strtok("");

	switch ($cmd) {

		case "rule":		mp_cmd_add_rule($arg2);		break;
		default:			mp_cmd_add_rule($arg);

	}

}

function mp_cmd_add_rule($arg) {

	global $ctl, $autocommit;
	
	$cmd = mp_parse_cmd($arg, array("T_SOURCE", "T_DESTINATION", "T_FWMARK", "T_TOS", "T_POLICY", "T_EXPIRES"), array("T_VIA"));
	
	$ctl->Add_Rule(mp_parsed_cmd_to_rule($cmd));

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_insert($arg) {

	$cmd = strtok($arg, " ");
	$arg2 = strtok("");

	switch ($cmd) {

		case "rule":		mp_cmd_insert_rule($arg2);		break;
		default:			mp_cmd_insert_rule($arg);

	}

}

function mp_cmd_insert_rule($arg) {

	global $ctl, $autocommit;
	
	$pos = strtok($arg, " ");

	if (is_numeric($pos)) {

		$position = (int)$pos;
		$arg2 = strtok("");

	} else {

		$position = 1;
		$arg2 = $arg;

	}
	
	$cmd = mp_parse_cmd($arg2, array("T_SOURCE", "T_DESTINATION", "T_FWMARK", "T_TOS", "T_POLICY", "T_EXPIRES"), array("T_VIA"));
	
	$ctl->Insert_Rule(mp_parsed_cmd_to_rule($cmd), $position);

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_del($arg) {

	$cmd = strtok($arg, " ");
	$arg2 = strtok("");

	switch ($cmd) {

		case "rule":		mp_cmd_del_rule($arg2);		break;
		default:			mp_cmd_del_rule($arg);

	}

}

function mp_cmd_del_rule($arg) {

	global $ctl, $autocommit;

	if (is_numeric($arg)) {

		$rules = array_values($ctl->Get_Rules());
		
		if (!isset($rules[$arg])) {

			throw new Exception("rule {$arg} not found");

		}
		
		$rule = array("match" => $rules[$arg]["match"], "value" => $rules[$arg]["value"]);
	
	} else {

		$cmd = mp_parse_cmd($arg, array("T_SOURCE", "T_DESTINATION", "T_FWMARK", "T_TOS"));

		$rule = mp_parsed_cmd_to_rule($cmd);
	
	}

	$ctl->Del_Rule($rule);

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_set_rule($arg) {

	global $ctl, $autocommit;

	$tmp = explode(" ", $arg);
	
	if (is_numeric($tmp[0])) {

		$rid = $tmp[0];
		
		$rules = array_values($ctl->Get_Rules());
		
		if (!isset($rules[$rid])) {

			throw new Exception("rule {$rid} not found");

		}
		
		array_shift($tmp);

		$arg = implode(" ", $tmp);
		
		$cmd = mp_parse_cmd($arg, array("T_POLICY", "T_EXPIRES"), array("T_VIA"));

		$rule = mp_parsed_cmd_to_rule($cmd);
		$rule["match"] = $rules[$rid]["match"];
		$rule["value"] = $rules[$rid]["value"];
	
	} else {

		$cmd = mp_parse_cmd($arg, array("T_DEFAULT", "T_SOURCE", "T_DESTINATION", "T_FWMARK", "T_TOS", "T_POLICY", "T_EXPIRES"), array("T_VIA"));

		$rule = mp_parsed_cmd_to_rule($cmd);
	
	}
	
	$ctl->Set_Rule($rule);

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_default($arg) {

	$cmd = strtok($arg, " ");
	$arg2 = strtok("");

	switch ($cmd) {

		case "rule":		mp_cmd_default_rule($arg2);
		default:			mp_cmd_default_rule($arg);

	}

}

function mp_cmd_default_rule($arg) {

	mp_cmd_set_rule("default {$arg}");

}

function mp_cmd_rule($arg) {

	$cmd = strtok($arg, " ");
	$arg2 = strtok("");

	switch ($cmd) {

		case "add":			mp_cmd_add_rule($arg2);			break;
		case "insert":		mp_cmd_insert_rule($arg2);		break;
		case "del":			mp_cmd_del_rule($arg2);			break;
		case "set":			mp_cmd_set_rule($arg2);			break;
		case "default":		mp_cmd_default_rule($arg2);		break;
		case "dump":		mp_cmd_dump_rule($arg2);		break;

		default:			mp_cmd_set_rule($arg);
	
	}

}

function mp_cmd_commit() {

	global $ctl, $console_level;

	list($tot_cmds, $warn) = $ctl->Commit_Rules();

	if ($console_level & Log::LEVEL_DEBUG) {
	
		echo "rules commited: {$tot_cmds} executed, {$warn} warning(s)\n";

	}

}

function mp_cmd_refresh() {

	global $ctl, $autocommit;

	$ctl->Refresh_Rules();

	if ($autocommit) {

		$ctl->Commit_Rules();

	}

}

function mp_cmd_dump($arg) {

	$cmd = strtok($arg, " ");
	$arg2 = strtok("");

	switch ($cmd) {

		case "rules":		mp_cmd_dump_rules($arg2);	break;
		case "rule":		mp_cmd_dump_rule($arg2);	break;
		case "default":		mp_cmd_dump_rule(0);		break;

		default:			mp_cmd_dump_rules($arg2);
	
	}

}

function _mp_dump_rule($rule, $prefix = "") {

	if (!is_array($rule)) {

		throw new Exception("cannot dump rule: invalid argument (not an array)");
	
	}

	$ret = $prefix . mp_rule_match_to_str($rule["match"]) . " ";

	foreach ($rule["value"] as $val) {

		if (is_int($val) || is_float($val)) {

			$ret .= $val;

		} else {

			$ret .= "\"{$val}\"";

		}
		
		$ret .= " ";
	
	}

	$ret .= "{\n";

	foreach ($rule["routes"] as $route) {

		$ret .= $prefix . "\t" . "via ";

		$tmp = array();
		
		foreach ($route as $gateway) {

			$tmp[] = "\"{$gateway}\"";
		
		}

		$ret .= implode(" ", $tmp) . ";\n";
	
	}
	
	if ($rule["default_policy"] !== Rule::DEF_DEFAULT) {

		$ret .= $prefix . "\t" . "policy \"" . mp_rule_policy_to_str($rule["default_policy"]) . "\";\n";
	
	}
	
	if (isset($rule["table"])) {

		$ret .= $prefix . "\t" . "table \"" . $rule["table"] . "\";\n";
	
	}
	
	$ret .= $prefix . "}\n";

	return $ret;

}

function mp_cmd_dump_rule($arg) {

	global $ctl;
	
	if (($arg === false) || ($arg === "")) {

		throw new Exception("dump rule: missing argument");
	
	} else if (is_numeric($arg)) {

		$rules = array_values($ctl->Get_Rules());
		
		if (!isset($rules[$arg])) {

			throw new Exception("rule {$arg} not found");

		}
		
		$rule = array("match" => $rules[$arg]["match"], "value" => $rules[$arg]["value"]);
	
	} else {

		$cmd = mp_parse_cmd($arg, array("T_SOURCE", "T_DESTINATION", "T_FWMARK", "T_TOS", "T_DEFAULT"));

		$rule = mp_parsed_cmd_to_rule($cmd);
	
	}
	
	$rule = current($ctl->Get_Rules($rule));
	
	echo _mp_dump_rule($rule);

}

function mp_cmd_dump_rules() {

	global $ctl;

	$ret = "rules {\n";

	foreach ($ctl->Get_Rules() as $rule) {

		$ret .= _mp_dump_rule($rule, "\t");
	
	}
	
	$ret .= "}\n";

	echo $ret;

}

function mp_cmd_reload() {

	global $ctl;

	return $ctl->Reload();

}

function mp_cmd_stop($arg) {

	global $ctl;

	$ctl->Stop_Daemon($arg);

}

function mp_start_daemon() {

	$daemon = dirname(__FILE__) . "/mpathd";

	if (is_executable($daemon)) {

		system($daemon, $ret);
	
	} else {

		echo "error: cannot start {$daemon}\n";

		$ret = -1;

	}

	return $ret;

}

$opts = getopt("hc:d:p:vVq-?");

if (isset($opts["?"])) {

	if (is_array($opts["?"])) {

		$param = $opts["?"][0];

	} else {

		$param = $opts["?"];

	}
	
	echo "error: invalid parameter '{$param}'\n";
	exit(-1);

}

if (isset($opts["h"])) {

	mp_show_help();
	exit();

}

if (isset($opts["v"])) {

	echo VERSION . "\n";
	exit();

}

$conffile = $opts["c"] ? $opts["c"] : (Conf::DEFAULT_PATH . DIRECTORY_SEPARATOR . DEFAULT_CONF_FILENAME);

try {

	$conf = Conf::Parse($conffile);

} catch (Tokenizer_Exception $e) {
	
	echo "config syntax error: {$e->getMessage()}\n";
	exit(-1);

} catch (Exception $e) {

	echo "error: {$e->getMessage()}\n";
	exit(-1);

}

$host = $opts["d"]
or $host = Client::DEFAULT_HOSTNAME;

$port = $opts["p"]
or $port = Client::DEFAULT_PORT;

if (isset($opts["-"])) {
	
	$opts["q"] = true;

	if ($opts["-"] === "start") {

		exit(mp_start_daemon());
		
	}

}

try {

	$ctl = new Client($host, $port);
	$ctl->options->agent = "mpath/" . VERSION;

	if (!isset($opts["-"])) {
	
		$ctl->events->on_Log_Message = function ($level, $message) {

			if (!($level & $GLOBALS["console_level"])) {

				return false;

			}
			
			switch ($level) {

				case Log::LEVEL_DEBUG:			$lvl = "DEBUG";		break;
				case Log::LEVEL_NOTICE:			$lvl = "NOTICE";	break;
				case Log::LEVEL_WARNING:		$lvl = "WARNING";	break;
				case Log::LEVEL_ERROR:			$lvl = "ERROR";		break;
				
				default:						$lvl = "??";		break;
			
			}

			echo date("Ymd:His") . " {$lvl}: {$message}\n";

		};

	}
	
	$ctl->events->on_Disconnect = function() {

		echo "lost connection to mpathd\n";
		exit(-3);

	};

	$ctl->Connect();

} catch (Exception $e) {

	echo "connect error: {$e->getMessage()}\n";
	exit(-2);

}

if (isset($opts["-"]) && ($opts["-"] === "restart")) {

	$ctl->Stop_Daemon("restarting");
	sleep(2);
	exit(mp_start_daemon());

}

if (isset($opts["V"])) {

	echo $ctl->server . "\n";
	exit();

}

if (!isset($opts["q"])) {
	
	echo "connected to {$ctl->server} at {$host}:{$port}\n";

}

$stdin = fopen("php://stdin", "r");

$console_level = Log::LEVEL_DEFAULT;
$autocommit = true;

$autoexec = array();

if (isset($opts["-"])) {

	// Get commands from command line
	
	if (is_array($opts["-"])) {

		$autoexec[] = implode(" ", $opts["-"]);

	} else {

		$autoexec[] = $opts["-"];

	}

	$autoexec[] = "exit";

} else {
	
	// Or fallback to console with .mpathrc
	
	$user = posix_getpwuid(posix_getuid());
	$rcfile = $user["dir"] . DIRECTORY_SEPARATOR . ".mpathrc";

	if (file_exists($rcfile)) {

		foreach (file($rcfile) as $tmp) {

			$tmp = rtrim($tmp, "\r\n");

			if (!$tmp) continue;
			
			$autoexec[] = $tmp;

		}

	}

}

if (!$autoexec) mp_prompt();

while (true) {

	// Handle user input

	$cmd_s = array_shift($autoexec)
	or $cmd_s = mp_read_input();
	
	if ($cmd_s !== false) {
		
		$cmd = strtok($cmd_s, " ");
		$arg = strtok("");

		try {

			switch ($cmd) {

				case "show":		mp_cmd_show($arg);				break;
				case "set":			mp_cmd_set($arg);				break;
				case "add":			mp_cmd_add($arg);				break;
				case "del":			mp_cmd_del($arg);				break;
				case "rule":		mp_cmd_rule($arg);				break;
				case "default":		mp_cmd_default($arg);			break;
				case "disengage":	mp_cmd_disengage($arg);			break;
				case "disable":		mp_cmd_disable($arg);			break;
				case "activate":	mp_cmd_activate($arg);			break;
				case "auto":		mp_cmd_auto($arg);				break;
				case "commit":		mp_cmd_commit($arg);			break;
				case "refresh":		mp_cmd_refresh($arg);			break;
				case "dump":		mp_cmd_dump($arg);				break;
				case "reload":		mp_cmd_reload($arg);			break;
				case "stop":		mp_cmd_stop($arg);				break;
				case "help":		mp_show_online_help($arg);		break;
				case "exit":		/* bye bye */					break 2;

				case "":			/* do nothing */				break;

				default:			echo "unkown command '{$cmd}', see 'help'\n";

			}
		
		} catch (Tokenizer_Exception $e) {

			echo "syntax error: {$e->getMessage()}\n";
			if (isset($opts["-"])) exit(-1);

		} catch (Exception $e) {

			echo "error: {$e->getMessage()}\n";
			if (isset($opts["-"])) exit(-4);
		
		}
		
		if (!$autoexec) mp_prompt();
	
	}

	if (!$autoexec) {
		
		stream_set_blocking($stdin, false);
		
		$ctl->Poll(1, array($stdin));

		stream_set_blocking($stdin, true);

	}

}

$ctl->Disconnect();

if (!isset($opts["q"])) {

	echo "disconnected\n";

}

?>
