<?php

/**
 *
 * mpath-tools - failover and load balancing between multiple ISP made easy
 * 
 * Copyright (C) 2008-2010 Vincent Negrier aka. sIX <six at aegis-corp.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 * @package mpath-tools
 * @subpackage www
 */

/**
 * Require the config file
 */
require "config.php";
require "mpath-tools/inc/client.php";

use MPath_Tools\Control\Client;

function sh4_size_format($filesize, $separator=" ", $unit="B") {

	if ($filesize >= 1099511627776) {

		$fs = $filesize / 1099511627776;
		$fu = "T".$unit;

	} else if ($filesize >= 1073741824) {

		$fs = $filesize / 1073741824;
		$fu = "G".$unit;

	} else if ($filesize >= 1048576) {

		$fs = $filesize / 1048576;
		$fu = "M".$unit;

	} else if ($filesize >= 1024) {

		$fs = $filesize / 1024;
		$fu = "K".$unit;

	} else {

		$fs = $filesize;
		$fu = $unit;

	}

	$ret = number_format($fs, (($fu != $unit) ? 1 : 0)).$separator.$fu;

	return $ret;

}


function ts_to_string($timestamp) {

        $up_d=(int)($timestamp/86400);
        $up_h=((int)($timestamp/3600))%24;
        $up_m=((int)($timestamp/60))%60;
        $up_s=$timestamp%60;

        $tstr=($up_d?($up_d."d, "):"").(($up_h||$up_d)?($up_h."h, "):"").(($up_m||$up_h||$up_d)?($up_m."m, "):"").$up_s."s";

        return ($tstr);

}

function get_traffic_data($interfaces) {

	$ret = array();
	
	foreach (file("/proc/net/dev") as $s) {

		$ifname = trim(strtok($s, ":"));

		if (!in_array($ifname, $interfaces))
			continue;

		$rx_bytes = strtok(" ");
		for ($a = 0; $a < 7; $a++) strtok(" ");
		$tx_bytes = strtok(" ");

		$ret[$ifname] = array($rx_bytes, $tx_bytes, microtime(true));
	
	}

	return $ret;

}

ob_start("ob_gzhandler");
header("Content-type: text/xml");
session_start();

try {

	$ctl = new Client(MPATH_WWW_CONTROL_ADDR, MPATH_WWW_CONTROL_PORT);
	$ctl->Connect();

	$hide_gws = explode(",", MPATH_WWW_HIDE_GATEWAYS);
	$show_gws = $traffic_ifs = array();
	
	$i = 0;
	
	foreach ($ctl->List_Gateways(false) as $gw) {

		if (!in_array($gw["name"], $hide_gws)) {

			$show_gws[] = $gw["name"];
			$traffic_ifs[$gw["name"]] = $gw["interface"];

		}

	}

	$status = $ctl->Get_Status();
	$gws = $ctl->Get_Gateways($show_gws);

	$mpathd_version = $status["version"];
	$mpathd_uptime = ts_to_string(time() - $status["started_t"]);
	
	if (MPATH_WWW_SHOW_TRAFFIC) {

		$traffic = array();
		$traffic_current = get_traffic_data($traffic_ifs);

		if (isset($_SESSION["traffic"])) {

			foreach ($show_gws as $k => $gwname) {

				$rx_diff = $traffic_current[$traffic_ifs[$gwname]][0] - $_SESSION["traffic"][$k][0];
				$tx_diff = $traffic_current[$traffic_ifs[$gwname]][1] - $_SESSION["traffic"][$k][1];
				$mt_diff = $traffic_current[$traffic_ifs[$gwname]][2] - $_SESSION["traffic"][$k][2];

				if ($rx_diff < 0) 
					$rx_diff = $traffic_current[$traffic_ifs[$gwname]][0];

				if ($tx_diff < 0) 
					$tx_diff = $traffic_current[$traffic_ifs[$gwname]][1];
				
				if ($mt_diff !== 0)
					$traffic[$k] = array(	"rx_bps"	=> $rx_diff / $mt_diff,
											"tx_bps"	=> $tx_diff / $mt_diff);
			
			}
			
		}

		$_SESSION["traffic"] = $_SESSION["traffic_n1"];
		
		foreach ($show_gws as $k => $gwname)
			$_SESSION["traffic_n1"][$k] = $traffic_current[$traffic_ifs[$gwname]];

	}

} catch (Exception $e) {

	echo "<div class='error'>";
	echo "error connecting to mpathd: {$e->getMessage()}";
	echo "</div>";

	die();

}

$stats_t_a = array(30, 300, 3600);
$stats_ids = array(	"sent"		=> "sent",
					"received"	=> "received",
					"error"		=> "error",
					"rtt min"	=> "rtt_min",
					"rtt avg"	=> "rtt_avg",
					"rtt max"	=> "rtt_max",
					"rtt mdev"	=> "rtt_mdev",
					"% loss"	=> "pct_loss",
					"% error"	=> "pct_error");


echo "<mpathdata>\n";
echo "	<value id='mpathd_version'>" . $mpathd_version . "</value>\n";
echo "	<value id='mpathd_uptime'>" . $mpathd_uptime . "</value>\n";

$svalues = array();

foreach ($gws as $k => $gw) {

	echo "	<value id='gw_{$k}_state' class='" . $gw["state"] . "'>" . $gw["state"] . "</value>\n";
	echo "	<value id='gw_{$k}_status'>" . $gw["status"] . "</value>\n";

	echo "	<data gw='{$k}' id='latency'>" . $gw["stats"][10]["rtt avg"] . "</data>\n";
	echo "	<data gw='{$k}' id='loss'>" . ($gw["stats"][10]["% loss"] + $gw["stats"][10]["% error"]) . "</data>\n";
	
	if (MPATH_WWW_SHOW_TRAFFIC) {

		echo "	<data gw='{$k}' id='traffic_in'>" . $traffic[$k]["rx_bps"] . "</data>\n";
		echo "	<data gw='{$k}' id='traffic_out'>" . $traffic[$k]["tx_bps"] . "</data>\n";
		echo "	<value id='gw_{$k}_traffic_in_str'>" . sh4_size_format($traffic[$k]["rx_bps"]) . "/s</value>\n";
		echo "	<value id='gw_{$k}_traffic_out_str'>" . sh4_size_format($traffic[$k]["tx_bps"]) . "/s</value>\n";
		echo "	<value id='gw_{$k}_traffic_in_str_bits'>" . sh4_size_format($traffic[$k]["rx_bps"] * 8, " ", "bit") . "/s</value>\n";
		echo "	<value id='gw_{$k}_traffic_out_str_bits'>" . sh4_size_format($traffic[$k]["tx_bps"] * 8, " ", "bit") . "/s</value>\n";
	
	}
	
	foreach ($stats_ids as $id => $xid) {

		foreach ($stats_t_a as $st) {

			$data = $gws[$k]["stats"][$st][$id];

			switch ($id) {

				case "rtt min":
				case "rtt avg":
				case "rtt max":
				case "rtt mdev":	

				if ($data < 100)
					$color = "green";
				else if ($data < 500)
					$color = "yellow";
				else
					$color = "red";

				$data = number_format($data, 2);
				
				break;


				case "% loss":
				case "% error": 
				
				if ($data == 0)
					$color = "green";
				else if ($data < "1")
					$color = "yellow";
				else
					$color = "red";
				
				$data = number_format($data, 2);

				break;


				default: $color = "green";
			
			}

			$svalues["gw_{$k}_{$xid}_{$st}"] = array($color, $data);
			
		}
	
	}

}

foreach ($svalues as $k => $v) {

	if ($_SESSION["svalues"][$k] !== $v)
		echo "	<value id='{$k}' class='stats_{$v[0]}'>{$v[1]}</value>\n";

}

$_SESSION["svalues"] = $svalues;

echo "</mpathdata>\n";

?>