<?php

/**
 *
 * mpath-tools - failover and load balancing between multiple ISP made easy
 * 
 * Copyright (C) 2008-2010 Vincent Negrier aka. sIX <six at aegis-corp.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 * @package mpath-tools
 * @subpackage www
 */

/**
 * Require the config file
 */
require "config.php";
require "mpath-tools/inc/client.php";
require "mpath-tools/inc/gateway.php";

use MPath_Tools\Control\Client;

session_start();
session_destroy();

try {

	$ctl = new Client(MPATH_WWW_CONTROL_ADDR, MPATH_WWW_CONTROL_PORT);
	$ctl->Connect();

} catch (Exception $e) {

	echo "<div class='error'>";
	echo "error connecting to mpathd: {$e->getMessage()}";
	echo "</div>";

	die();

}

?>
<!DOCTYPE html>
<html>
<head>
<title><?=MPATH_WWW_PAGE_TITLE?></title>
<link rel="stylesheet" type="text/css" href="style.css" />
<meta http-equiv="Content-Type" content="text/html;charset=utf-8" />
<script src="monitor.js" type="text/javascript"></script>
<!--[if IE]><script language="javascript" type="text/javascript" src="inc/excanvas.min.js"></script><![endif]-->
<script language="javascript" type="text/javascript" src="inc/jquery.js"></script>
<script language="javascript" type="text/javascript" src="inc/jquery.flot.js"></script>
<?php

if ($_REQUEST["show_probe"] || $_REQUEST["show_traffic"]) {

	echo "<script type='text/javascript'>\n";
	echo "default_display_probe = " . ($_REQUEST["show_probe"] ? "true" : "false") . ";\n";
	echo "default_display_traffic = " . ($_REQUEST["show_traffic"] ? "true" : "false") . ";\n";
	echo "</script>\n";

}

?>
</head>
<body>
<div id='main'>

<div id='header'>
<h2 id='header_text'><?=MPATH_WWW_PAGE_TITLE?></h2>
<table id='daemon_data'>
<tr>
<td><span class='label'>version</span></td>
<td><span class='value' id='mpathd_version'></span></td>
</tr>
<tr>
<td><span class='label'>uptime</span></td>
<td><span class='value' id='mpathd_uptime'></span></td>
</tr>
</table>
</div>

<div id='gateways'>
<?php

try {

	$hide_gws = explode(",", MPATH_WWW_HIDE_GATEWAYS);
	$show_gws = $traffic_ifs = array();
	
	foreach ($ctl->List_Gateways(false) as $gw) {

		if (!in_array($gw["name"], $hide_gws)) 
			$show_gws[] = $gw["name"];

		$traffic_ifs[$gw["name"]] = $gw["interface"];
	
	}

	$stats_t_a = array(30, 300, 3600);
	$stats_ids = array(	"sent"		=> "sent",
						"received"	=> "received",
						"error"		=> "error",
						"rtt min"	=> "rtt_min",
						"rtt avg"	=> "rtt_avg",
						"rtt max"	=> "rtt_max",
						"rtt mdev"	=> "rtt_mdev",
						"% loss"	=> "pct_loss",
						"% error"	=> "pct_error");
	
	$gws = $ctl->Get_Gateways($show_gws);
	
	foreach ($show_gws as $k => $gwname) {
		
		echo "<div class='gateway'>";

		echo "<div class='header'>";
		echo "<div class='display_options'>";

		$options = array("<span class='display_options' data-gw='{$k}' data-opt='probe'></span>");

		if (MPATH_WWW_SHOW_TRAFFIC)
			$options[] = "<span class='display_options' data-gw='{$k}' data-opt='traffic'></span>";

		echo implode(" - ", $options);
		
		echo "</div>";
		echo "<span class='gwname'>{$gwname}</span>";
		echo " - ";
		echo "<span class='value' id='gw_{$k}_state'></span>";
		echo "&nbsp;";
		echo "<span class='label' id='gw_{$k}_status'></span>";
		echo "</div>";

		echo "<div style='clear:both'></div>";

		echo "<div class='gw_element' id='gw_{$k}_probe'>";
		echo "<div class='stats'>";
		echo "<table class='stats'>";
		echo "<thead>";
		echo "<tr>";
		echo "<th>&nbsp;</th>";
		echo "<th>30 sec</th>";
		echo "<th>5 min</th>";
		echo "<th>1 hour</th>";
		echo "</tr>";
		echo "</thead>";
		echo "<tbody>";

		foreach ($stats_ids as $id => $xid) {

			echo "<tr>";
			echo "<td class='label'>{$id}</td>";

			foreach ($stats_t_a as $st) {

				echo "<td><span id='gw_{$k}_{$xid}_{$st}'></span></td>";
			
			}

			echo "</tr>";
		
		}

		echo "</tbody>";
		echo "</table>";
		echo "</div>";
		echo "<div id='graph_latency_{$k}' class='graph'></div>";
		echo "<div style='clear:both'></div>";
		echo "</div>";

		if (MPATH_WWW_SHOW_TRAFFIC) {
		
			echo "<div class='gw_element' id='gw_{$k}_traffic'>";
			echo "<div class='stats'>";
			echo "<table class='traffic_data'>";
			echo "<tr>";
			echo "<td><span class='label'>Inbound</span></td>";
			echo "<td><span id='gw_{$k}_traffic_in_str' class='value inbound'></span><br /><span id='gw_{$k}_traffic_in_str_bits' class='value inbound bits'></span></td>";
			echo "</tr>";
			echo "<tr>";
			echo "<td><span class='label'>Outbound</span></td>";
			echo "<td><span id='gw_{$k}_traffic_out_str' class='value outbound'></span><br /><span id='gw_{$k}_traffic_out_str_bits' class='value outbound bits'></span></td>";
			echo "</tr>";
			echo "</table>";
			echo "</div>";
			echo "<div id='graph_traffic_{$k}' class='graph traffic'></div>";
			echo "<div style='clear:both'></div>";
			echo "</div>";
		
		}
		
		echo "</div>";
	
	}

} catch (Exception $e) {

	echo "error: {$e->getMessage()}";
	die();

}

?>
</div>

<div id='footer'>
mpath realtime web/ajax monitor is part of mpath-tools/<?=\MPath_Tools\VERSION?> (c) 2008-2010 by sIX - <a href="http://mpath-tools.optilian.org">http://mpath-tools.optilian.org</a>
</div>

</div>
</body>
</html>